@@ -1,5 +1,6 @@
 /* 
  * Copyright (C) 2006, Intel Corporation
+ * Copyright (C) 2012, Neil Horman <nhorman@tuxdriver.com> 
  * 
  * This file is part of irqbalance
  *
@@ -45,6 +46,7 @@ struct load_balance_info {
 	unsigned int num_within;
 	unsigned int num_over;
 	unsigned int num_under;
+	unsigned int num_powersave;
 	struct topo_obj *powersave;
 };
 
@@ -93,8 +95,7 @@ static void move_candidate_irqs(struct irq_info *info, void *data)
 
 	*remaining_deviation -= info->load;
 
-	if (debug_mode)
-		printf("Selecting irq %d for rebalancing\n", info->irq);
+	log(TO_CONSOLE, LOG_INFO, "Selecting irq %d for rebalancing\n", info->irq);
 
 	migrate_irq(&info->assigned_obj->interrupts, &rebalance_irq_list, info);
 
@@ -106,13 +107,16 @@ static void migrate_overloaded_irqs(struct topo_obj *obj, void *data)
 	struct load_balance_info *info = data;
 	int deviation;
 
+	if (obj->powersave_mode)
+		info->num_powersave++;
+
 	/*
  	 * Don't rebalance irqs on objects whos load is below the average
  	 */
 	if (obj->load <= info->avg_load) {
 		if ((obj->load + info->std_deviation) <= info->avg_load) {
 			info->num_under++;
-			if (!info->powersave)
+			if (power_thresh != ULONG_MAX && !info->powersave)
 				if (!obj->powersave_mode)
 					info->powersave = obj;
 		} else
@@ -160,6 +164,7 @@ static void clear_powersave_mode(struct topo_obj *obj, void *data __attribute__(
 	int ___load_sources;\
 	memset(&(info), 0, sizeof(struct load_balance_info));\
 	for_each_object((name), gather_load_stats, &(info));\
+	(info).load_sources = ((info).load_sources == 0) ? 1 : ((info).load_sources);\
 	(info).avg_load = (info).total_load / (info).load_sources;\
 	for_each_object((name), compute_deviations, &(info));\
 	___load_sources = ((info).load_sources == 1) ? 1 : ((info).load_sources - 1);\
@@ -172,14 +177,14 @@ void update_migration_status(void)
 {
 	struct load_balance_info info;
 	find_overloaded_objs(cpus, info);
-	if (cycle_count > 5) {
+	if (power_thresh != ULONG_MAX && cycle_count > 5) {
 		if (!info.num_over && (info.num_under >= power_thresh) && info.powersave) {
-			syslog(LOG_INFO, "cpu %d entering powersave mode\n", info.powersave->number);
+			log(TO_ALL, LOG_INFO, "cpu %d entering powersave mode\n", info.powersave->number);
 			info.powersave->powersave_mode = 1;
 			if (g_list_length(info.powersave->interrupts) > 0)
 				for_each_irq(info.powersave->interrupts, force_irq_migration, NULL);
-		} else if (info.num_over) {
-			syslog(LOG_INFO, "Load average increasing, re-enabling all cpus for irq balancing\n");
+		} else if ((info.num_over) && (info.num_powersave)) {
+			log(TO_ALL, LOG_INFO, "Load average increasing, re-enabling all cpus for irq balancing\n");
 			for_each_object(cpus, clear_powersave_mode, NULL);
 		}
 	}
@@ -203,7 +208,8 @@ void reset_counts(void)
 
 static void dump_workload(struct irq_info *info, void *unused __attribute__((unused)))
 {
-	printf("Interrupt %i node_num %d (class %s) has workload %lu \n", info->irq, irq_numa_node(info)->number, classes[info->class], (unsigned long)info->load);
+	log(TO_CONSOLE, LOG_INFO, "Interrupt %i node_num %d (class %s) has workload %lu \n",
+	    info->irq, irq_numa_node(info)->number, classes[info->class], (unsigned long)info->load);
 }
 
 void dump_workloads(void)
